﻿using System;
using System.IO;

namespace SamplePlugin.Lib
{
    /// <summary>
    /// Sets the working directory to specified directory until disposed.
    /// </summary>
    public class DirectoryContext : IDisposable
    {
        private readonly string _OriginalDirectory;

        /// <summary>
        /// Initializes a new instance of the <see cref="DirectoryContext"/> class.
        /// </summary>
        /// <param name="path">The path.</param>
        public DirectoryContext(DirectoryInfo path)
        {
            if (!path.Exists)
            {
                throw new ArgumentException("path");
            }
            _OriginalDirectory = Environment.CurrentDirectory;
            Environment.CurrentDirectory = path.FullName;
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            Environment.CurrentDirectory = _OriginalDirectory;
        }
    }
}
