﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using EnvDTE;

using Microsoft.VisualStudio.CommandBars;

using Shimmy.Api;
using Shimmy.Api.Utils;

namespace SamplePlugin
{
    public class SimpleCommand : CommandActionBase
    {
        private CommandMenu _CommandMenu;

        /// <summary>
        /// Gets the name of the action.
        /// </summary>
        /// <value>The name.</value>
        public override string ActionName
        {
            get { return "Simple Command"; }
        }

        /// <summary>
        /// Gets or sets the button caption.
        /// </summary>
        /// <value>The button caption.</value>
        [Description("The button name to display")]
        public string ButtonCaption { get; set; }

        /// <summary>
        /// Gets the names of the commands handled by this handler
        /// </summary>
        /// <value></value>
        public override string[] CommandNames
        {
            get { return new[] { "MyCommand" }; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SimpleCommand"/> class.
        /// </summary>
        public SimpleCommand()
        {
            ButtonCaption = "Open Plugin Directory";
            Enabled = false;
        }

        /// <summary>
        /// Initializes the action.
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        public override void Initialize(PluginContext pluginContext)
        {
            var codeWindow = (CommandBar)pluginContext.Application.CommandBars[CommandBarNames.Code_Window];
            _CommandMenu = new CommandMenu(codeWindow, pluginContext);
            _CommandMenu.SubMenu("Shimmy")
                        .Command("MyCommand", ButtonCaption, icon: Properties.Resources.shimmy)
                        .EndMenu();
        }


        public override void Unload(PluginContext pluginContext)
        {
            // We can recieve an unload without an initialize
            if (_CommandMenu != null) 
            {
                _CommandMenu.Delete();
            }
        }

        /// <summary>
        /// This is called when the command's availability is updated
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        /// <param name="commandName">Name of the command.</param>
        /// <param name="neededText">Text that is needed for the command.</param>
        /// <param name="status">The state of the command in the user interface.</param>
        /// <param name="commandText">Text requested by the neededText parameter.</param>
        public override void QueryStatus(PluginContext pluginContext, string commandName, vsCommandStatusTextWanted neededText, ref vsCommandStatus status, ref object commandText)
        {
            status = vsCommandStatus.vsCommandStatusEnabled | vsCommandStatus.vsCommandStatusSupported;
        }

        /// <summary>
        /// This is called when the command is invoked.
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        /// <param name="commandName">Name of the command.</param>
        /// <param name="executeOption">Describes how the command should be run.</param>
        /// <param name="varIn">Parameters passed from the caller to the command handler.</param>
        /// <param name="varOut">Parameters passed from the command handler to the caller.</param>
        /// <param name="handled">Informs the caller if the command was handled or not.</param>
        /// <seealso cref="QueryStatus"/>
        public override void Exec(PluginContext pluginContext, string commandName, vsCommandExecOption executeOption, ref object varIn, ref object varOut, ref bool handled)
        {
            System.Diagnostics.Process.Start(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "\\Shimmy\\Plugins"));

        }
    }
}
