﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

using EnvDTE;

using EnvDTE80;

using Shimmy.Api;
using Shimmy.Api.Utils;

using VSLangProj;

namespace SamplePlugin
{
    public class T4ProcessorAction : ActionBase
    {
        /// <summary>
        /// Gets or sets the application.
        /// </summary>
        /// <value>The application.</value>
        protected DTE2 Application { get; set; }

        /// <summary>
        /// Gets the name of the action.
        /// </summary>
        /// <value>The name.</value>
        public override string ActionName
        {
            get { return "T4 Processor"; }
        }

        /// <summary>
        /// Gets or sets the name of the T4 template.
        /// </summary>
        /// <value>The name of the T4 template.</value>
        [Description("The name of the T4 template (e.g., T4CSS.tt)")]
        public string T4TemplateName { get; set; }

        /// <summary>
        /// Gets or sets the name of the T4 template.
        /// </summary>
        /// <value>The name of the T4 template.</value>
        [Description("The regular expression pattern to match files that will trigger this action.  Action is triggered when the document(s) are saved.")]
        public string RegexPattern { get; set; }
        
        /// <summary>
        /// Initializes a new instance of the <see cref="T4ProcessorAction"/> class.
        /// </summary>
        public T4ProcessorAction()
        {
            T4TemplateName = "T4CSS.tt";
            RegexPattern =  @"\.less";
            Enabled = false;
        }

        /// <summary>
        /// Initializes the action.
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        public override void Initialize(PluginContext pluginContext)
        {
            // Get a reference to the application
            Application = pluginContext.Application;

            // Need to copy to member variable otherwise events will be disconnected
            pluginContext.ApplicationEvents.DocumentEvents.DocumentSaved += DocumentSaved;
        }

        /// <summary>
        /// Unloads this action.  Clean up any resources/events.
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        public override void Unload(PluginContext pluginContext)
        {
            pluginContext.ApplicationEvents.DocumentEvents.DocumentSaved -= DocumentSaved;
        }

        private void DocumentSaved(Document document)
        {
            if (Regex.IsMatch(document.Name, RegexPattern, RegexOptions.IgnoreCase))
            {
                try
                {
                    var t4Template = (Application.Solution.FindProjectItem(T4TemplateName).Object as VSProjectItem);
                    if (t4Template != null && MessageBox.Show("Run corresponding T4 template?", ActionName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question) == DialogResult.Yes)
                    {
                        t4Template.RunCustomTool();
                    }
                }
                catch (Exception e)
                {
                    ErrorHandler.HandleError("Error running T4 template: " + e, e);
                }
            }
        }
    }
}
