using System;
using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Linq;

using EnvDTE;

namespace Shimmy.Api
{
    /// <summary>
    /// The base action from which all command-based actions should derive
    /// </summary>
    [InheritedExport(typeof(ICommandHandler))]
    public abstract class CommandActionBase : ActionBase, ICommandHandler
    {
        /// <summary>
        /// Gets the names of the commands handled by this handler
        /// </summary>
        /// <value></value>
        [Browsable(false)]
        public abstract string[] CommandNames { get; }

        /// <summary>
        /// Initializes a new instance of the <see cref="CommandActionBase"/> class.
        /// </summary>
        protected CommandActionBase()
        {
            Enabled = true;
        }
        
        /// <summary>
        /// This is called when the command's availability is updated
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        /// <param name="commandName">Name of the command.</param>
        /// <param name="neededText">Text that is needed for the command.</param>
        /// <param name="status">The state of the command in the user interface.</param>
        /// <param name="commandText">Text requested by the neededText parameter.</param>
        public abstract void QueryStatus( PluginContext pluginContext, string commandName, vsCommandStatusTextWanted neededText, ref vsCommandStatus status, ref object commandText );

        /// <summary>
        /// This is called when the command is invoked.
        /// </summary>
        /// <param name="pluginContext">The plugin context.</param>
        /// <param name="commandName">Name of the command.</param>
        /// <param name="executeOption">Describes how the command should be run.</param>
        /// <param name="varIn">Parameters passed from the caller to the command handler.</param>
        /// <param name="varOut">Parameters passed from the command handler to the caller.</param>
        /// <param name="handled">Informs the caller if the command was handled or not.</param>
        /// <seealso cref="QueryStatus"/>
        public abstract void Exec( PluginContext pluginContext, string commandName, vsCommandExecOption executeOption, ref object varIn, ref object varOut, ref bool handled );
    }
}