﻿using System;

using Microsoft.VisualStudio.CommandBars;

namespace Shimmy.Api.Utils
{
    internal static class CommandBarUtils
    {
        /// <summary>
        /// Creates a new submenu at the top of the <paramref name="source"/>.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="caption">The caption.</param>
        /// <param name="consolidateMenus">if set to <c>true</c> when a menu is added and it already exists, the existing one will be used.</param>
        /// <returns></returns>
        public static CommandBarPopup PrependSubMenu(CommandBar source, string caption, bool consolidateMenus = true)
        {
            return CreateSubMenu(source, caption, 1, consolidateMenus);
        }

        /// <summary>
        /// ACreates a new submenu at the bottom of the <paramref name="source"/>.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="caption">The caption.</param>
        /// <param name="consolidateMenus">if set to <c>true</c> when a menu is added and it already exists, the existing one will be used.</param>
        /// <returns></returns>
        public static CommandBarPopup AppendSubMenu(CommandBar source, string caption, bool consolidateMenus = true)
        {
            return CreateSubMenu( source, caption, source.Controls.Count + 1, consolidateMenus );
        }

        private static CommandBarPopup CreateSubMenu( CommandBar source, string caption, int position, bool consolidateMenus )
        {
            if (consolidateMenus)
            {
                // Search for an existing commandbar control with the same caption
                foreach (CommandBarControl control in source.Controls)
                {
                    if (control.Caption == caption)
                    {
                        return (CommandBarPopup)control;
                    }
                }
            }

            var submenu = (CommandBarPopup)source.Controls.Add(MsoControlType.msoControlPopup, Type.Missing, Type.Missing, position, true);
            submenu.Caption = caption;
            return submenu;
        }
    }
}
