﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using EnvDTE;

using EnvDTE80;

namespace Shimmy.Lib.Dte
{
    /// <summary>
    /// The default MiscFileEvents class does not expose events of interest.  Each
    /// "feature" has it's own ProjectItemsEvents - so we wire them all up.
    /// <remarks>
    /// Search for AutomationEvents here HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\VisualStudio\10.0\Packages
    /// to find more names.
    /// http://www.mztools.com/articles/2005/MZ2005009.aspx
    /// </remarks>
    /// </summary>
    internal class ProjectItemsEventsWrapper : ProjectItemsEvents
    {
        private readonly DTE2 _Application;
        private readonly IList<ProjectItemsEvents> _ProjectItemsEventsList;
        private static readonly string[] _NamedEvents = new[] { "VBProjectItemsEvents", "CSharpProjectItemsEvents", "WebSiteItemsEvents" };

        public event _dispProjectItemsEvents_ItemAddedEventHandler ItemAdded
        {
            add
            {
                foreach (var projectItemsEvents in _ProjectItemsEventsList)
                {
                    projectItemsEvents.ItemAdded += value;
                }
            }
            remove
            {
                foreach (var projectItemsEvents in _ProjectItemsEventsList)
                {
                    projectItemsEvents.ItemAdded -= value;
                }
            }
        }

        public event _dispProjectItemsEvents_ItemRemovedEventHandler ItemRemoved
        {
            add
            {
                foreach (var projectItemsEvents in _ProjectItemsEventsList)
                {
                    projectItemsEvents.ItemRemoved += value;
                }
            }
            remove
            {
                foreach (var projectItemsEvents in _ProjectItemsEventsList)
                {
                    projectItemsEvents.ItemRemoved -= value;
                }
            }
        }

        public event _dispProjectItemsEvents_ItemRenamedEventHandler ItemRenamed
        {
            add
            {
                foreach (var projectItemsEvents in _ProjectItemsEventsList)
                {
                    projectItemsEvents.ItemRenamed += value;
                }
            }
            remove
            {
                foreach (var projectItemsEvents in _ProjectItemsEventsList)
                {
                    projectItemsEvents.ItemRenamed -= value;
                }
            }
        }

        public ProjectItemsEventsWrapper(DTE2 application)
        {
            _Application = application;
            _ProjectItemsEventsList = new List<ProjectItemsEvents>();
            foreach (var namedEvent in _NamedEvents)
            {
                AddEventSource(namedEvent);
            }
        }

        private void AddEventSource(string name)
        {
            try
            {
                var appEvent = _Application.Events.GetObject(name);
                if (appEvent != null)
                {
                    _ProjectItemsEventsList.Add((ProjectItemsEvents)appEvent);
                }
            }
            catch (Exception)
            {
            }
        }
    }
}
