﻿namespace Shimmy.Lib
{
    internal sealed class GlobalSettings
    {
        /// <summary>
        /// Gets or sets the configuration folder locations, newline delimited.
        /// </summary>
        /// <value>The configuration folder locations.</value>
        public string ConfigurationDirectoryLocations { get; set; }

        /// <summary>
        /// Gets the configuration directories.
        /// </summary>
        /// <value>The configuration directories.</value>
        public string[] ConfigurationDirectories
        {
            get
            {
                if (string.IsNullOrEmpty(ConfigurationDirectoryLocations))
                {
                    return null;
                }
                return ConfigurationDirectoryLocations.Split('\r', '\n');
            }
        }

        /// <summary>
        /// Creates a <see cref="GlobalSettings"/> object that is pre-loaded.
        /// </summary>
        public static GlobalSettings Load()
        {
            return SettingsManager.Instance.Restore(new GlobalSettings());
        }

        private GlobalSettings()
        {
        }

        /// <summary>
        /// Saves the settings.
        /// </summary>
        public void Save()
        {
            SettingsManager.Instance.Persist(this);
        }
    }
}
