﻿using System.Collections.Generic;

using Microsoft.Win32;

using Shimmy.Lib.Utils;

namespace Shimmy.Lib
{
    internal class SettingsManager
    {
        private static readonly SettingsManager _Instance = new SettingsManager();

        /// <summary>
        /// Gets the instance.
        /// </summary>
        /// <value>The instance.</value>
        public static SettingsManager Instance { get { return _Instance; } }

        /// <summary>
        /// Persists the specified object to the backing store.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="objectToPersist">The object to persist.</param>
        /// <param name="category">The category.</param>
        /// <returns></returns>
        public T Persist<T>(T objectToPersist, string category = null)
        {
            var keyName = GetKeyPath(objectToPersist.GetType().FullName, category);
            var registryKey = Registry.CurrentUser.CreateSubKey(keyName);
            var objectValues = ReflectionUtils.ObjectToDictionary(objectToPersist);
            foreach (var objectValue in objectValues)
            {
                registryKey.SetValue(objectValue.Key, objectValue.Value, RegistryValueKind.String);
            }
            return objectToPersist;
        }

        /// <summary>
        /// Restores the specified object to restore.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="objectToRestore">The object to restore.</param>
        /// <param name="category">The category.</param>
        /// <returns></returns>
        public T Restore<T>(T objectToRestore, string category = null)
        {
            var keyName = GetKeyPath(objectToRestore.GetType().FullName, category);
            var registryKey = Registry.CurrentUser.CreateSubKey(keyName);

            IDictionary<string, string> dictionary = GetValueDictionary(registryKey);
            ReflectionUtils.DictionaryToObject(objectToRestore, dictionary);
            return objectToRestore;
        }

        private static IDictionary<string, string> GetValueDictionary(RegistryKey registryKey)
        {
            IDictionary<string, string> dictionary = new Dictionary<string, string>();
            var valueNames = registryKey.GetValueNames();
            foreach (var valueName in valueNames)
            {
                dictionary.Add(valueName, registryKey.GetValue(valueName).ToString());
            }
            return dictionary;
        }
        
        /// <summary>
        /// Gets the key path.
        /// </summary>
        /// <value>The key path.</value>
        private static string GetKeyPath(string typeName, string category)
        {
            var path = PluginEnvironment.Constants.SETTINGS_REG_KEY + "\\" + typeName + "\\";
            if (!string.IsNullOrEmpty(category))
            {
                path += category + "\\";
            }
            return path;
        }
    }
}
