﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Reflection;

using Shimmy.Api;
using Shimmy.Api.Config;

namespace Shimmy.Lib.Utils
{
    internal static class ReflectionUtils
    {
        public static IDictionary<string, string> ObjectToDictionary(object source)
        {
            var type = source.GetType();
            var properties = type.GetProperties();
            IDictionary<string, string> result = new Dictionary<string, string>(properties.Length);
            foreach (var propertyInfo in properties)
            {
                if (IsValidProperty(propertyInfo))
                {
                    var value = propertyInfo.GetValue(source, null);
                    var attributes = propertyInfo.GetCustomAttributes(typeof(PropertySerializerAttribute), true);
                    if (attributes.Length == 1)
                    {
                        var serializer = Activator.CreateInstance((attributes[0] as PropertySerializerAttribute).Serializer) as IPropertySerializer;
                        if (serializer != null)
                        {
                            value = serializer.Serialize(value);
                        }
                    }
                    result.Add(propertyInfo.Name, Convert.ToString(value));
                }
            }
            return result;
        }

        public static void DictionaryToObject(object source, IDictionary<string, string> dictionary)
        {
            var type = source.GetType();
            var properties = type.GetProperties();
            IDictionary<string, object> result = new Dictionary<string, object>(properties.Length);
            foreach (var propertyInfo in properties)
            {
                if (dictionary.ContainsKey(propertyInfo.Name))
                {
                    if (IsValidProperty(propertyInfo))
                    {
                        string stringValue = dictionary[propertyInfo.Name];
                        object value = stringValue;
                        if (typeof(IConvertible).IsAssignableFrom(propertyInfo.PropertyType))
                        {
                            value = Convert.ChangeType(value, propertyInfo.PropertyType);
                        }
                        else
                        {
                            var attributes = propertyInfo.GetCustomAttributes(typeof(PropertySerializerAttribute), true);
                            if (attributes.Length == 1)
                            {
                                var serializer = Activator.CreateInstance((attributes[0] as PropertySerializerAttribute).Serializer) as IPropertySerializer;
                                if (serializer != null)
                                {
                                    value = serializer.Deserialize(stringValue);
                                }
                            }
                        }
                        propertyInfo.SetValue(source, value, null);
                    }
                }
            }
        }

        private static bool IsValidProperty(PropertyInfo propertyInfo)
        {
            // Must be browsable
            var browsableAttributes = propertyInfo.GetCustomAttributes(typeof(BrowsableAttribute), true);
            bool isBrowsable = browsableAttributes.Length == 0 || (browsableAttributes[0] as BrowsableAttribute).Browsable;
            return isBrowsable && propertyInfo.CanWrite && (typeof(IConvertible).IsAssignableFrom(propertyInfo.PropertyType) || propertyInfo.GetCustomAttributes(typeof(PropertySerializerAttribute), true).Length > 0);
        }
    }
}
